#include <u.h>
#include <libc.h>
#include <draw.h>
#include <bio.h>
#include <ndb.h>

typedef struct Colmap Colmap;
struct Colmap {
	char *name;
	ulong value;
};

static Colmap coltab[] = {
	{"default",		DNotacolor},
	{"DBlack",		DBlack},
	{"DWhite", 		DWhite},
	{"DRed",		DRed},
	{"DGreen",		DGreen},
	{"DBlue",		DBlue},
	{"DCyan",		DCyan},
	{"DMagenta",		DMagenta},
	{"DYellow",		DYellow},
	{"DPaleyellow",		DPaleyellow},
	{"DDarkyellow",		DDarkyellow},
	{"DDarkgreen",		DDarkgreen},
	{"DPalegreen",		DPalegreen},
	{"DMedgreen",		DMedgreen},
	{"DDarkblue",		DDarkblue},
	{"DPalebluegreen",	DPalebluegreen},
	{"DPaleblue",		DPaleblue},
	{"DBluegreen",		DBluegreen},
	{"DGreygreen",		DGreygreen},
	{"DPalegreygreen",	DPalegreygreen},
	{"DYellowgreen",	DYellowgreen},
	{"DMedblue",		DMedblue},
	{"DGreyblue",		DGreyblue},
	{"DPalegreyblue",	DPalegreyblue},
	{"DPurpleblue",		DPurpleblue},
};

static ulong
name2col(char *name)
{
	int i;

	for(i=0; i<nelem(coltab); i++){
		if(strcmp(name, coltab[i].name) == 0)
			return coltab[i].value;
	}

	return DNotacolor;
}

static char *tname;
static Ndb *tdb;
static QLock tlock;

void
themeinit(char *theme)
{
	char *home, *dbpath;

	qlock(&tlock);
	home = nil;
	if((tname = strdup(theme)) == nil){
		fprint(2, "themeinit strdup: %r");
		goto out;
	}
	if((home = getenv("home")) != nil){
		dbpath = smprint("%s/lib/theme", home);
		if(dbpath == nil){
			fprint(2, "themeinit smprint: %r");
			goto out;
		}

		tdb = ndbopen(dbpath);
		free(dbpath);
	}
	if(tdb == nil)
		tdb = ndbopen("/lib/theme");

out:
	if(home != nil)
		free(home);

	qunlock(&tlock);
}

void
themeend(void)
{
	qlock(&tlock);
	free(tname);
	tname = nil;
	if(tdb != nil){
		ndbclose(tdb);
		tdb = nil;
	}
	qunlock(&tlock);
}

static ulong
convcolor(char *s)
{
	char *p;
	ulong n;

	n = strtoul(s, &p, 0);
	if(s != p && *p == '\0')
		return n;

	return name2col(s);
}

ulong
themeget(char *st, ulong def)
{
	ulong n;
	Ndbs ds;
	char *p;

	qlock(&tlock);
	p = nil;
	n = def;
	if(tdb == nil)
		goto done;

	p = ndbgetvalue(tdb, &ds, "theme", tname, st, nil);
	if(p != nil){
		n = convcolor(p);
		if(n == DNotacolor)
			n = def;

		goto done;
	}

	p = ndbgetvalue(tdb, &ds, "theme", "default", st, nil);
	if(p != nil){
		n = convcolor(p);
		if(n == DNotacolor)
			n = def;

		goto done;
	}

done:
	if(p != nil)
		free(p);
	qunlock(&tlock);

	return n;
}
